#include "string.h"
#include "windows.h"
#include "banner.h"

                            // color defines
#define CLR_WHITE   0x0FFFFFFL
#define CLR_BLACK   0x0L
#define CLR_DKGRAY  0x0808080L
#define CLR_BLUE    0x0FF0000L
                            // defines banner appearance
                            // app name color
#define CLR_NAME    CLR_BLUE
                            // border edge in pixels
#define BORDER        8
               // name text height multiple of standard font
#define NAME_FONT_MULTIPLE    3

static HICON hIcon;            
static LPSTR lpAppName, lpMsg1, lpMsg2;
static RECT rIcon, rAppName, rMsg1, rMsg2;
static HFONT hf;
static HWND hBanner;

static HFONT CreateNameFont(HDC hDC) {
    LOGFONT lf;
    TEXTMETRIC tm;
    GetTextMetrics(hDC,&tm);
    memset (&lf, 0, sizeof (LOGFONT));
    lf.lfCharSet    = ANSI_CHARSET;
    lf.lfPitchAndFamily = VARIABLE_PITCH | FF_ROMAN;
    lf.lfHeight     = NAME_FONT_MULTIPLE*tm.tmHeight;
    lf.lfQuality    = PROOF_QUALITY;           
    lstrcpy(lf.lfFaceName,"Times New Roman Bold Italic");
    return CreateFontIndirect(&lf);
}
static void Paint3DText(HDC hDC, LPSTR lpstr, RECT r,
        COLORREF cr1, COLORREF cr2) {
    COLORREF crOld  = SetTextColor(hDC,cr2);
    int nOldMode    = SetBkMode(hDC, TRANSPARENT);
    DrawText(hDC,lpstr,-1,&r,DT_CENTER|DT_NOCLIP);
    OffsetRect(&r,-1,-1);
    SetTextColor(hDC,cr1);
    DrawText(hDC,lpstr,-1,&r,DT_CENTER|DT_NOCLIP);
    SetTextColor(hDC,crOld);
    SetBkMode(hDC, nOldMode);
}
static void Paint3DRect(HDC hDC, RECT r) {
    HBRUSH hBrush;
    HPEN hPen = SelectObject(hDC,GetStockObject(WHITE_PEN));
    hBrush = SelectObject(hDC,GetStockObject(NULL_BRUSH));
    Rectangle(hDC,r.left,r.top,r.right,r.bottom);
    OffsetRect(&r,-1,-1);
    SelectObject(hDC,CreatePen(PS_SOLID,1,CLR_DKGRAY));
    Rectangle(hDC,r.left,r.top,r.right,r.bottom);
    DeleteObject(SelectObject(hDC,hPen));
    SelectObject(hDC,hBrush);
}
static void PositionObjects(int nWidth, int nHeight) {
    int x, y, dely = 0;
    y = nHeight - max(rIcon.bottom, rAppName.bottom) -
        rMsg1.bottom - rMsg2.bottom;
    y /= 2 + (int)(lpMsg1 != NULL) + (int)(lpMsg2 != NULL);
    if(rAppName.bottom < rIcon.bottom)
        dely = (rIcon.bottom - rAppName.bottom)/2;
    x = (nWidth - rAppName.right)/2;
    OffsetRect(&rAppName,x + BORDER,y + dely + BORDER);
    if(hIcon) {
        x = (x - rIcon.right)/2;
        if((rAppName.bottom-rAppName.top) > rIcon.bottom)
            dely = ((rAppName.bottom-rAppName.top) -
                rIcon.bottom)/2;
        else dely = 0;
        OffsetRect(&rIcon,x+BORDER,y+dely+BORDER);
    }
    if(lpMsg1) {
        OffsetRect(&rMsg1,BORDER + (nWidth-rMsg1.right)/2,
            max(rAppName.bottom,rIcon.bottom) + y);
    }
    if(lpMsg2) {
        if(lpMsg1) dely = rMsg1.bottom;
        else dely = max(rAppName.bottom,rIcon.bottom);
        OffsetRect(&rMsg2,BORDER + (nWidth-rMsg2.right)/2,
            y + dely);
    }
}
static void InitDlg(HWND hDlg) {
    RECT rc, r;
    HFONT hfOld;
    int nWidth, nHeight, n2Cx, n2Cy, n4Brdr = 4*BORDER;
    HDC hDC = GetDC(hDlg);
    GetWindowRect(GetDesktopWindow(),&rc);
    nWidth = (rc.right/2) - n4Brdr -
        (n2Cx = 2*GetSystemMetrics(SM_CXDLGFRAME));
    nHeight = (rc.bottom/2) - n4Brdr -
        (n2Cy = 2*GetSystemMetrics(SM_CYDLGFRAME));
    SetRectEmpty(&rAppName);
    SetRectEmpty(&rMsg1);
    SetRectEmpty(&rMsg2);
    if((hf=CreateNameFont(hDC)) != NULL)
        hfOld   = SelectObject(hDC,hf);
    DrawText(hDC,lpAppName,-1,&rAppName,DT_CALCRECT);
    if(hf) SelectObject(hDC,hfOld);
    if(hIcon) SetRect(&rIcon,0,0,32,32);
    else SetRectEmpty(&rIcon);
    if(lpMsg1) DrawText(hDC,lpMsg1,-1,&rMsg1,DT_CALCRECT);
    if(lpMsg2) DrawText(hDC,lpMsg2,-1,&rMsg2,DT_CALCRECT);
    nWidth = max(nWidth,
              max(2*(rIcon.right+BORDER) + rAppName.right,
                max(rMsg1.right, rMsg2.right)));
    nHeight = max(nHeight,max(rIcon.bottom,rAppName.bottom)+
                    rMsg1.bottom + rMsg2.bottom + 2*BORDER);
    SetRect(&r,0,0,nWidth+n2Cx+n4Brdr,nHeight+n2Cy+n4Brdr);
    PositionObjects(nWidth+2*BORDER,nHeight+2*BORDER);
    MoveWindow(hDlg,(rc.right-r.right)/2,
        (rc.bottom-r.bottom)/2,r.right,r.bottom,FALSE);
    ReleaseDC(hDlg, hDC);
}
#ifdef __BORLANDC__
    #pragma argsused
#endif
BOOL CALLBACK PASCAL _export BannerDlgProc(HWND hDlg,
                   UINT msg, WPARAM wParam, LPARAM lParam) {
    RECT r;
    switch(msg) {
    case WM_INITDIALOG:
        InitDlg(hDlg);
        return TRUE;
    case WM_PAINT: {
        PAINTSTRUCT ps;
        HFONT hfOldFont;
        HDC hDC = BeginPaint(hDlg,&ps);
        GetClientRect(hDlg,&r);
        InflateRect(&r,-BORDER,-BORDER);
        Paint3DRect(hDC,r);
        if(hf) hfOldFont = SelectObject(hDC,hf);
        Paint3DText(hDC,lpAppName,rAppName,CLR_NAME,
            CLR_BLACK);
        if(hf) SelectObject(hDC,hfOldFont);
        if(lpMsg1)
            Paint3DText(hDC,lpMsg1,rMsg1,
                                      CLR_BLACK,CLR_DKGRAY);
        if(lpMsg2)
            Paint3DText(hDC,lpMsg2,rMsg2,
                                      CLR_DKGRAY,CLR_WHITE);
        if(hIcon)
            DrawIcon(hDC,rIcon.left,rIcon.top,hIcon);
        EndPaint(hDlg,&ps);
        }
        return TRUE;
    case WM_ERASEBKGND:
    case WM_ICONERASEBKGND:
        if(GetUpdateRect(hDlg,&r,FALSE))
            FillRect((HDC)wParam,&r,
                      (HBRUSH)GetStockObject(LTGRAY_BRUSH));
        return TRUE;
    case WM_DESTROY:
        if(hf)
            DeleteObject(hf);
        hBanner = (HWND) NULL;
        return TRUE;
    }
    return FALSE;
}
static HGLOBAL CreateDlgTemplate(void) {
    HGLOBAL hDlgTmplt;
    LPSTR lpdt;
    long lStyle = WS_POPUP | WS_VISIBLE | WS_DLGFRAME;
    hDlgTmplt = GlobalAlloc(GHND,16);
    if(hDlgTmplt && ((lpdt=(LPSTR)
                           GlobalLock(hDlgTmplt))!= NULL)) {
        _fmemcpy(lpdt,(LPSTR)&lStyle,sizeof(long));
        GlobalUnlock(hDlgTmplt);
    }
    return hDlgTmplt;
}
BOOL OpenBanner(HWND hWnd, HICON hicon, LPSTR lpappname,
            LPSTR lpmsg1, LPSTR lpmsg2) {
    HINSTANCE  hInst = (HINSTANCE)
                         GetWindowWord(hWnd, GWW_HINSTANCE);
    HGLOBAL hdt;
    void FAR* pdt;
    if(lpappname) {
        if(!hBanner) {
            hIcon = hicon; lpAppName = lpappname;
            lpMsg1 = lpmsg1; lpMsg2 = lpmsg2;
            if((hdt=CreateDlgTemplate()) != NULL) {
                if((pdt=GlobalLock(hdt)) != NULL) {
                  hBanner = CreateDialogIndirect(hInst,pdt,
                     hWnd,BannerDlgProc);
                  GlobalUnlock(hdt);
                }
                GlobalFree(hdt);
            }
        }
    }
    return hBanner != NULL;
}

void CloseBanner(void) {
    if(hBanner) DestroyWindow(hBanner);
}
